'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var accesscontrol_1 = require("accesscontrol");
var CrudAccessControl = /** @class */ (function () {
    function CrudAccessControl(resource, grants) {
        this.resource = resource;
        this.grants = grants;
        this.ac = new accesscontrol_1.AccessControl(grants);
    }
    CrudAccessControl.prototype.check = function (user, op) {
        var _ac = this.ac;
        //check for user any
        var permission;
        logger_1.default.trace("check can " + user.roles + " " + op + "Any for " + this.resource);
        var anyPermission = this.doCheck(_ac, user.roles, op, 'Any', this.resource);
        if (anyPermission.granted) {
            anyPermission.type = 'any';
            anyPermission.for = 'user';
            permission = anyPermission;
        }
        else {
            //check for user own
            logger_1.default.trace("check can " + user.roles + " " + op + "Own for " + this.resource);
            var ownPermission = this.doCheck(_ac, user.roles, op, 'Own', this.resource);
            if (ownPermission.granted) {
                ownPermission.type = 'own';
                ownPermission.for = 'user';
                permission = ownPermission;
            }
            else {
                //check for organization own
                var orgRoles = this.getOrgRoles(user);
                logger_1.default.trace("check can " + orgRoles + " " + op + "Own for " + this.resource);
                var permission = this.doCheck(_ac, orgRoles, op, 'Own', this.resource);
                if (permission.granted) {
                    permission.type = 'own';
                    permission.for = 'organization';
                }
                permission.allRoles = orgRoles;
                permission.allRoles.push(user.roles);
                permission = permission;
            }
        }
        logger_1.default.trace(JSON.stringify(permission), permission);
        if (!permission.granted) {
            logger_1.default.trace(_ac.getGrants());
        }
        ;
        return permission;
    };
    CrudAccessControl.prototype.doCheck = function (_ac, roles, op, type, resource) {
        var permission;
        try {
            permission = _ac.can(roles)[op + type](resource);
        }
        catch (err) {
            logger_1.default.warn(err.message);
            permission = { granted: false };
        }
        return permission;
    };
    ;
    CrudAccessControl.prototype.getOrgRoles = function (user) {
        var roles = [];
        if (user.organization && user.organization.ref
            && user.organization.ref.code) {
            var org = user.organization;
            roles.push(org.ref.code + ":" + org.role);
        }
        return roles;
    };
    return CrudAccessControl;
}());
exports.CrudAccessControl = CrudAccessControl;
